@@ -1,59 +1,86 @@
+// Copyright 2018 Stefan Kroboth
+//
+// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
+// http://apache.org/licenses/LICENSE-2.0> or the MIT license <LICENSE-MIT or
+// http://opensource.org/licenses/MIT>, at your option. This file may not be
+// copied, modified, or distributed except according to those terms.
+
 //! Macros for persistently caching function calls
 //!
-//! The values are cached either in files or on Redis. Two storages, `FileStorage` and `RedisStorage`
-//! are provided.
+//! The values are cached either in files or on Redis. Two storages, `FileStorage` and
+//! `RedisStorage` are provided.
 //! Caching is performed based on the function name and function parameters, meaning that for every
 //! combination of function and parameters, the returned value is stored in a storage. Subsequent
 //! calls of this function with the same parameters are not computed, but instead fetched from the
 //! storage. This can lead to an decrease in computing time in case the function call is
 //! computationally more expensive than fetching the value from the storage. The storages are
 //! persistent (stored on disk) and can be shared between different threads and processes.
-//! All Parameters to the function need to be Hashable. The return value needs to be serializeable
-//! by the crate `bincode`.
+//! All parameters of the function to be cached need to be Hashable. The return value needs to be
+//! serializeable by the crate `bincode`.
+//!
+//! There are two different ways of caching:
+//!
+//! 1) Caching individual function calls with the `cache!` macro. This way the function can still
+//!    be used without caching if necessary.
+//! 2) Caching every function call. For this, either the `cache_func!` macro or a procedural macro
+//!    can be used. The latter is achieved with the compiler directive `#[persistent_cache]` of the
+//!    subcrate `persistentcache_procmacro`.
 //!
 //! # Setup
 //!
-//! Add the following dependencies to your projet:
+//! Add the following dependencies to your project:
 //!
 //! ```text
 //! [dependencies]
 //! lazy_static = "*"
-//! bincode = "*"
 //! persistentcache = "*"
+//! persistentcache_procmacro = "*"  # Only needed for `#[peristent_cache]`
 //! ```
 //!
-//! # Caching function calls with `cache!`
+//! # Caching a function with `#[persistent_cache]`
 //!
-//! The macro `cache!` caches a function call. The advantage of this approach over the macro
-//! `cache_func` is that different storages can be used for different calls. Furthermore the
-//! function can still be called without caching if desired.
-//! However, in case of recursive functions, this will most likely not work as expected because the
-//! recursive calls will not be cached.
-//! The macro expects the function to return a value of type `Result<T, Box<std::error::Error>>`.
+//! The easiest way to cache all calls to a function is by preceding it with the
+//! `#[persistent_cache]` directive. This modifies the function such that values never computed
+//! before are computed and cached in a storage. Already computed values are fetched from said
+//! storage without computing. The return type needs to implement the `Serializable` trait.
 //!
 //! ## Example
 //!
 //! ```
-//! #![allow(redundant_closure_call)]
-//! extern crate bincode;
-//! #[macro_use] extern crate persistentcache;
+//! #[macro_use]
+//! extern crate lazy_static;
+//! #[macro_use]
+//! extern crate persistentcache;
+//! extern crate persistent_procmacro;
 //! use persistentcache::*;
+//! use persistentcache::storage::{FileStorage, RedisStorage};
+//! use persistentcache_procmacro::persistent_cache;
 //!
-//! fn add_two(a: u64) -> Result<u64, Box<std::error::Error>> {
+//! // Either store it in a `FileStorage`...
+//! #[persistent_cache]
+//! #[params(FileStorage, "test_dir")]
+//! fn add_two_file(a: u64) -> u64 {
 //!     println!("Calculating {} + 2...", a);
-//!     Ok(a + 2)
+//!     a + 2
+//! }
+//!
+//! // ... or in a `RedisStorage`
+//! #[persistent_cache]
+//! #[params(RedisStorage, "redis://127.0.0.1")]
+//! fn add_two_redis(a: u64) -> u64 {
+//!     println!("Calculating {} + 2...", a);
+//!     a + 2
 //! }
 //!
 //! fn main() {
-//!     let s = storage::redis::RedisStorage::new("redis://127.0.0.1").unwrap();
 //!     // Function is called and will print "Calculating 2 + 2..." and "4"
-//!     println!("{}", cache!(s, add_two(2)).unwrap());
+//!     println!("{}", s, add_two_file(2));
 //!     // Value will be cached from Redis, will only print "4"
-//!     println!("{}", cache!(s, add_two(2)).unwrap());
+//!     println!("{}", s, add_two_file(2));
 //!     // Function is called and will print "Calculating 3 + 2..." and "5"
-//!     println!("{}", cache!(s, add_two(3)).unwrap());
+//!     println!("{}", s, add_two_redis(3));
 //!     // Value will be cached from Redis, will only print "5"
-//!     println!("{}", cache!(s, add_two(3)).unwrap());
+//!     println!("{}", s, add_two_redis(3));
 //! }
 //! ```
 //!
@@ -77,16 +104,14 @@
 //! storage.
 //! The advantage of this approach over `cache!` is that the function is modified and hence every
 //! call to the function will automatically take care of the caching. Furthermore it works with
-//! recursive calls. However, caching cannot be 'turned off' anymore.
-//! No assumption about the return type are made in this case. The function returns the same type
-//! as the initial function definition.
+//! recursive calls. However, caching cannot be disabled anymore.
+//! The return value needs to implemend the `Serializable` trait.
 //!
 //! ## Example
 //!
 //! ```
 //! #[macro_use] extern crate lazy_static;
 //! #[macro_use] extern crate persistentcache;
-//! extern crate bincode;
 //! use persistentcache::*;
 //!
 //! // Either store it in a `FileStorage`...
@@ -104,14 +129,14 @@
 //! });
 //!
 //! fn main() {
-//!     /*// Function is called and will print "Calculating 2 + 2..." and "4"
-//!     println!("{}", s, add_two_file(2));
+//!     // Function is called and will print "Calculating 2 + 2..." and "4"
+//!     println!("{}", add_two_file(2));
 //!     // Value will be cached from Redis, will only print "4"
-//!     println!("{}", s, add_two_file(2));
+//!     println!("{}", add_two_file(2));
 //!     // Function is called and will print "Calculating 3 + 2..." and "5"
-//!     println!("{}", s, add_two_redis(3));
+//!     println!("{}", add_two_redis(3));
 //!     // Value will be cached from Redis, will only print "5"
-//!     println!("{}", s, add_two_redis(3));*/
+//!     println!("{}", add_two_redis(3));
 //! }
 //! ```
 //!
@@ -126,6 +151,55 @@
 //! 5
 //! ```
 //!
+//!
+//! # Caching function calls with `cache!`
+//!
+//! The macro `cache!` caches a function call. The advantage of this approach over the macro
+//! `cache_func!` and the procedual macro `#[peristent_cache]` is that different storages can be
+//! used for different calls. Furthermore the function can still be called without caching if
+//! desired.
+//! However, in case of recursive functions, this will most likely not work as expected because the
+//! recursive calls will not be cached.
+//! The macro expects the function to return a value of type `Result<T, Box<std::error::Error>>`.
+//!
+//! ## Example
+//!
+//! ```
+//! #![allow(redundant_closure_call)]
+//! #[macro_use]
+//! extern crate persistentcache;
+//! use persistentcache::*;
+//!
+//! fn add_two(a: u64) -> u64 {
+//!     println!("Calculating {} + 2...", a);
+//!     a + 2
+//! }
+//!
+//! fn main() {
+//!     let s = storage::redis::RedisStorage::new("redis://127.0.0.1").unwrap();
+//!     // Function is called and will print "Calculating 2 + 2..." and "4"
+//!     println!("{}", cache!(s, add_two(2)));
+//!     // Value will be cached from Redis, will only print "4"
+//!     println!("{}", cache!(s, add_two(2)));
+//!     // Function is called and will print "Calculating 3 + 2..." and "5"
+//!     println!("{}", cache!(s, add_two(3)));
+//!     // Value will be cached from Redis, will only print "5"
+//!     println!("{}", cache!(s, add_two(3)));
+//! }
+//! ```
+//!
+//! This will print:
+//!
+//! ```text
+//! Calculating 2 + 2...
+//! 4
+//! 4
+//! Calculating 3 + 2...
+//! 5
+//! 5
+//! ```
+//!
+//!
 //! # Implementing other storages
 //!
 //! Storages need to implement the `PersistentCache` trait.
@@ -145,26 +219,29 @@
 //! This crate is inspired by [owls-cache](https://github.com/havoc-io/owls-cache) and its primary
 //! goal is to teach myself Rust. While working on it, I realised that a similar crate already
 //! exists: [cached-rs](https://github.com/jaemk/cached). I've borrowed a couple of ideas from
-//! there. Have a look at it, it looks much more professional than this crate and almost certainly
-//! has better developers. Unfortunately it lacks the 'persistent' part and the caches cannot be
-//! shared between processes/threads, but it should be fairly easy to extend it.
+//! there.  I suggest you have a look at the cached-rs crate, too.  Unfortunately it lacks the
+//! 'persistent' part and the caches cannot be shared between processes/threads, but it should be
+//! fairly easy to extend it. Furthermore, the excellent
+//! [accel](https://github.com/termoshtt/accell) has been very helpful. I shamelessly copied parts
+//! of it for the `persistentcache_procmacro` crate.
 //!
 #![recursion_limit = "1024"]
-#![cfg_attr(feature="clippy", feature(plugin))]
-#![cfg_attr(feature="clippy", plugin(clippy))]
-#![cfg_attr(feature="clippy", allow(redundant_closure_call))]
-// #![feature(trace_macros)]
-// #![feature(log_syntax)]
+#![cfg_attr(feature = "clippy", feature(plugin))]
+#![cfg_attr(feature = "clippy", plugin(clippy))]
+#![cfg_attr(feature = "clippy", allow(redundant_closure_call))]
 #![allow(unused_imports)]
 #![warn(missing_docs)]
+#![feature(proc_macro)]
 #[macro_use]
 extern crate error_chain;
+extern crate fs2;
 #[macro_use]
 extern crate lazy_static;
-extern crate bincode;
+extern crate persistentcache_procmacro;
 extern crate redis;
 extern crate regex;
-extern crate fs2;
+
+use persistentcache_procmacro::persistent_cache;
 
 mod errors {
     error_chain!{
@@ -172,7 +249,6 @@ mod errors {
             Redis(::redis::RedisError);
             Regex(::regex::Error);
             IO(::std::io::Error);
-            Bincode(::bincode::Error);
         }
     }
 }
@@ -202,44 +278,47 @@ pub trait PersistentCache {
 mod tests {
     extern crate num;
     use super::*;
+    use std;
     use std::error::Error;
     use self::num::{Num, NumCast};
     use storage::redis::RedisStorage;
     use storage::file::FileStorage;
+    use persistentcache_procmacro::persistent_cache;
 
-    fn test_func_1<T: Num + NumCast>(a: T, counter: &mut i64) -> Result<T> {
+    fn test_func_1<T: Num + NumCast>(a: T, counter: &mut i64) -> T {
         *counter += 1;
         let ten: T = NumCast::from(10_i64).unwrap();
-        Ok(a * ten)
+        a * ten
     }
 
-    fn test_func_2<T: Num>(a: T, b: T, counter: &mut i64) -> Result<T> {
+    fn test_func_2<T: Num>(a: T, b: T, counter: &mut i64) -> T {
         *counter += 1;
-        Ok(a * b)
+        a * b
     }
 
-    fn test_func_3<T: Copy>(a: &[T], counter: &mut i64) -> Result<Vec<T>> {
+    fn test_func_3<T: Copy>(a: &[T], counter: &mut i64) -> Vec<T> {
         *counter += 1;
-        Ok(vec![a[1], a[0]])
+        vec![a[1], a[0]]
     }
 
-    fn throw_error() -> Result<()> {
-        Err(
-            ::std::io::Error::new(::std::io::ErrorKind::Other, "fu").into(),
-        )
+    fn panic() -> () {
+        panic!("nothing");
     }
 
     #[test]
     fn test_fib() {
         let s = RedisStorage::new("redis://127.0.0.1").unwrap();
         s.flush().unwrap();
-        cache_func!(Redis, "redis://127.0.0.1", 
+        cache_func!(
+            Redis,
+            "redis://127.0.0.1",
             fn fib(n: u64) -> u64 {
-                if n == 0 || n ==1 {
-                    return n
+                if n == 0 || n == 1 {
+                    return n;
                 }
-                fib(n-1) + fib(n-2)
-            });
+                fib(n - 1) + fib(n - 2)
+            }
+        );
         assert_eq!(fib(10), 55);
         s.flush().unwrap();
     }
@@ -248,26 +327,64 @@ mod tests {
     fn test_func() {
         let s = FileStorage::new("file_test").unwrap();
         s.flush().unwrap();
-        cache_func!(File, "test", 
+        cache_func!(
+            File,
+            "test",
             fn add_two(n: u64) -> u64 {
                 n + 2
-            });
+            }
+        );
         assert_eq!(12, add_two(10));
         s.flush().unwrap();
     }
 
+    #[test]
+    fn test_func_procmacro() {
+        let s = FileStorage::new("file_test").unwrap();
+        s.flush().unwrap();
+
+        #[persistent_cache]
+        #[params(FileStorage, "file_test")]
+        fn add_two(n: u64) -> u64 {
+            n + 2
+        }
+
+        assert_eq!(12, add_two(10));
+        s.flush().unwrap();
+    }
+
+    #[test]
+    fn test_func_procmacro2() {
+        let s = FileStorage::new("file_test").unwrap();
+        s.flush().unwrap();
+        let mut counter: i64 = 0;
+
+        #[persistent_cache]
+        #[params(FileStorage, "file_test")]
+        fn test_func_proc(a: &Vec<i64>, counter: &mut i64) -> Vec<i64> {
+            *counter += 1;
+            vec![a[1], a[0]]
+        }
+
+        assert_eq!(vec![1, 2], test_func_proc(&vec![2, 1], &mut counter));
+        assert_eq!(counter, 1);
+        assert_eq!(vec![1, 2], test_func_proc(&vec![2, 1], &mut counter));
+        assert_eq!(counter, 2);
+        s.flush().unwrap();
+    }
+
     #[test]
     fn test_redis_storage() {
         let a: i64 = 6;
         let mut counter: i64 = 0;
         let s = RedisStorage::new("redis://127.0.0.1").unwrap();
         s.flush().unwrap();
-        assert_eq!(a * 10, test_func_1(a, &mut counter).unwrap());
+        assert_eq!(a * 10, test_func_1(a, &mut counter));
         assert_eq!(counter, 1);
-        assert_eq!(a * 10, cache!(s, test_func_1(a, &mut counter)).unwrap());
+        assert_eq!(a * 10, cache!(s, test_func_1(a, &mut counter)));
         assert_eq!(counter, 2);
         let mut counter: i64 = 1;
-        assert_eq!(a * 10, cache!(s, test_func_1(a, &mut counter)).unwrap());
+        assert_eq!(a * 10, cache!(s, test_func_1(a, &mut counter)));
         assert_eq!(counter, 1);
         s.flush().unwrap();
     }
@@ -278,12 +395,12 @@ mod tests {
         let mut counter: i64 = 0;
         let s = FileStorage::new("file_test").unwrap();
         s.flush().unwrap();
-        assert_eq!(a * 10, test_func_1(a, &mut counter).unwrap());
+        assert_eq!(a * 10, test_func_1(a, &mut counter));
         assert_eq!(counter, 1);
-        assert_eq!(a * 10, cache!(s, test_func_1(a, &mut counter)).unwrap());
+        assert_eq!(a * 10, cache!(s, test_func_1(a, &mut counter)));
         assert_eq!(counter, 2);
         let mut counter: i64 = 1;
-        assert_eq!(a * 10, cache!(s, test_func_1(a, &mut counter)).unwrap());
+        assert_eq!(a * 10, cache!(s, test_func_1(a, &mut counter)));
         assert_eq!(counter, 1);
         s.flush().unwrap();
     }
@@ -296,10 +413,10 @@ mod tests {
         let mut counter: i64 = 0;
         let s = FileStorage::new("file_test").unwrap();
         s.flush().unwrap();
-        assert_eq!(a * b, cache!(s, test_func_2(a, b, &mut counter)).unwrap());
+        assert_eq!(a * b, cache!(s, test_func_2(a, b, &mut counter)));
         assert_eq!(counter, 1);
         let mut counter: i64 = 0;
-        assert_eq!(a * b, cache!(s, test_func_2(b, a, &mut counter)).unwrap());
+        assert_eq!(a * b, cache!(s, test_func_2(b, a, &mut counter)));
         assert_eq!(counter, 1);
     }
 
@@ -309,18 +426,12 @@ mod tests {
         let mut counter: i64 = 0;
         let s = FileStorage::new("file_test").unwrap();
         s.flush().unwrap();
-        assert_eq!(vec![2, 1], test_func_3(&a, &mut counter).unwrap());
+        assert_eq!(vec![2, 1], test_func_3(&a, &mut counter));
         assert_eq!(counter, 1);
-        assert_eq!(
-            vec![2, 1],
-            cache!(s, test_func_3(&a, &mut counter)).unwrap()
-        );
+        assert_eq!(vec![2, 1], cache!(s, test_func_3(&a, &mut counter)));
         assert_eq!(counter, 2);
         let mut counter: i64 = 1;
-        assert_eq!(
-            vec![2, 1],
-            cache!(s, test_func_3(&a, &mut counter)).unwrap()
-        );
+        assert_eq!(vec![2, 1], cache!(s, test_func_3(&a, &mut counter)));
         assert_eq!(counter, 1);
         s.flush().unwrap();
     }
@@ -330,6 +441,6 @@ mod tests {
     fn failing_function() {
         let s = FileStorage::new("file_test").unwrap();
         s.flush().unwrap();
-        cache!(s, throw_error()).unwrap();
+        cache!(s, panic());
     }
 }
