@@ -1,3 +1,10 @@
+// Copyright 2018 Stefan Kroboth
+//
+// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
+// http://apache.org/licenses/LICENSE-2.0> or the MIT license <LICENSE-MIT or
+// http://opensource.org/licenses/MIT>, at your option. This file may not be
+// copied, modified, or distributed except according to those terms.
+
 //! Implementation of the macros `cache!` and `cache_func!`.
 use PREFIX;
 use PersistentCache;
@@ -38,7 +45,9 @@ macro_rules! cache_func {
     };
     // internal
     ($f:ident($($x:ident),*), $b:block, $prefix:expr) => {
-        use bincode;
+        extern crate bincode as pers_f_bincode;
+        // use bincode as pers_f_bincode;
+        // use bincode;
         use ::std::hash::{Hash, Hasher};
 
         let mut s = ::std::collections::hash_map::DefaultHasher::new();
@@ -51,10 +60,10 @@ macro_rules! cache_func {
         match result.len() {
             0 => {
                 let res = {$b};
-                S.lock().unwrap().set(&var_name, &bincode::serialize(&res, bincode::Infinite).unwrap()).unwrap();
+                S.lock().unwrap().set(&var_name, &pers_f_bincode::serialize(&res).unwrap()).unwrap();
                 return res;
             },
-            _ => return bincode::deserialize(&result).unwrap(),
+            _ => return pers_f_bincode::deserialize(&result).unwrap(),
         }
     }
 }
@@ -69,7 +78,7 @@ macro_rules! cache {
     // prefix provided
     ($storage:ident, $func:ident($($x:expr),*), $prefix:expr) => {
         (||{
-            use bincode;
+            extern crate bincode as pers_bincode;
             use ::std::hash::{Hash, Hasher};
 
             let mut s = ::std::collections::hash_map::DefaultHasher::new();
@@ -78,21 +87,29 @@ macro_rules! cache {
             }
             let var_name = format!("{}_{}_{}_{:?}", PREFIX, $prefix, stringify!($func), s.finish());
 
-            let result: Vec<u8> = $storage.get(&var_name)?;
+            let result: Vec<u8> = $storage.get(&var_name).unwrap();
+            let res;
             match result.len() {
                 0 => {
-                    match $func($($x),*) {
-                        Ok(res) => {
-                            $storage.set(&var_name, &bincode::serialize(&res, bincode::Infinite)?)?;
-                            Ok(res)
-                        }
-                        Err(e) => Err(e)
-                    }
+                    res = $func($($x),*);
+                    $storage.set(&var_name, &pers_bincode::serialize(&res).unwrap()).unwrap();
+                    res
+                    // match $func($($x),*) {
+                    //     Ok(res) => {
+                    //         $storage.set(&var_name, &pers_bincode::serialize(&res)?)?;
+                    //         Ok(res)
+                    //     }
+                    //     Err(e) => Err(e)
+                    // }
                 },
-                _ => match bincode::deserialize(&result) {
-                    Ok(res) => Ok(res),
-                    Err(e) => Err(e.into()), // I have no idea what I am doing.
-                }
+                // _ => match pers_bincode::deserialize(&result) {
+                //     Ok(res) => res,
+                //     Err(e) => panic!(e.into()), // I have no idea what I am doing.
+                // }
+                _ => {
+                    res = pers_bincode::deserialize(&result).unwrap();
+                    res
+                } 
             }
        })()
     }
