@@ -1,3 +1,10 @@
+// Copyright 2018 Stefan Kroboth
+//
+// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
+// http://apache.org/licenses/LICENSE-2.0> or the MIT license <LICENSE-MIT or
+// http://opensource.org/licenses/MIT>, at your option. This file may not be
+// copied, modified, or distributed except according to those terms.
+
 //! Storage for persistently saving return values of functions in Redis.
 use std::error::Error;
 use redis::{self, Commands};
@@ -7,7 +14,6 @@ use errors::*;
 use PREFIX;
 use PersistentCache;
 
-
 /// `RedisStorage` struct holds a `redis::Connection` variable.
 pub struct RedisStorage {
     con: redis::Connection,
@@ -29,7 +35,7 @@ impl RedisStorage {
     pub fn new(host: &str) -> Result<Self> {
         let client = redis::Client::open(host)?;
         let con = client.get_connection()?;
-        Ok(RedisStorage { con: con })
+        Ok(RedisStorage { con })
     }
 }
 
@@ -52,9 +58,9 @@ impl PersistentCache for RedisStorage {
 
     /// Delete all variables stored in the Redis database which start with `PREFIX_`.
     fn flush(&self) -> Result<()> {
-        let iter: redis::Iter<String> = redis::cmd("KEYS").arg(format!("{}_*", PREFIX)).iter(
-            &self.con,
-        )?;
+        let iter: redis::Iter<String> = redis::cmd("KEYS")
+            .arg(format!("{}_*", PREFIX))
+            .iter(&self.con)?;
         let cmd: &mut redis::Cmd = &mut redis::cmd("DEL");
         // Not a very good looking hack, but I dont know how to figure out whether the iterator is
         // empty or not...
